package com.akamai.androidmdt.androidsdkdemo;

import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ScrollView;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;

import com.akamai.amp.ads.AdsCount;
import com.akamai.amp.ads.AdsHelper;
import com.akamai.amp.ads.IAdsComponentListener;
import com.akamai.amp.ads.ima.AmpDAIManager;
import com.akamai.amp.ads.ima.AmpIMAManager;
import com.akamai.amp.ads.ima.GoogleAdsInfo;
import com.akamai.amp.ads.ima.IMA;
import com.akamai.amp.media.VideoPlayerContainer;
import com.akamai.amp.media.VideoPlayerView;
import com.akamai.amp.media.elements.MediaResource;
import com.akamai.amp.media.errors.ErrorType;
import com.akamai.amp.uimobile.generic.media.PlayerControlsOverlay;
import com.akamai.amp.utils.LogManager;
import com.google.ads.interactivemedia.v3.api.BaseManager;
import com.google.ads.interactivemedia.v3.api.ImaSdkFactory;
import com.google.ads.interactivemedia.v3.api.ImaSdkSettings;

import java.util.HashMap;
import java.util.Map;


public class MainActivity extends AppCompatActivity
        implements IAdsComponentListener<GoogleAdsInfo>,
        VideoPlayerContainer.VideoPlayerContainerCallback, LogManager.Logs {


    private static final String TAG = MainActivity.class.getName();
    private String AMP_LICENSE = "t_SEBeOhcpLarm8Vs5q2e_nbDUifOB8qEv7AdZVbaO03BOOhR9GoI7SrbV7_kXpZVuw3It4VRpPq0iYgkFR40w==";

    private String ADS_URL =
            "https://pubads.g.doubleclick.net/gampad/ads?sz=640x480&iu=/124319096/external/ad_rule_samples&ciu_szs=300x250&ad_rule=1&impl=s&gdfp_req=1&env=vp&output=vmap&unviewed_position_start=1&cust_params=deployment%3Ddevsite%26sample_ar%3Dpremidpostpod&cmsid=496&vid=short_onecue&correlator=";

    private VideoPlayerContainer mVideoContainer;
    private VideoPlayerView mVideoView;
    private AmpIMAManager csaiManager;
    private AmpDAIManager daiManager;
    private PlayerControlsOverlay mPlayerControlsOverlay;

    private ScrollView scrollView;
    private TextView logger;
    private TextView lastEventLog;
    private MediaResource mediaResource;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        mPlayerControlsOverlay = (PlayerControlsOverlay) findViewById(R.id.playerControls);
        setInlineUI();

        mVideoContainer = (VideoPlayerContainer) findViewById(R.id.playerViewCtrl);
        logger = (TextView) findViewById(R.id.logger);
        lastEventLog = (TextView) findViewById(R.id.last_event_log);
        scrollView = (ScrollView) findViewById(R.id.scrollView);
        mVideoContainer.addVideoPlayerContainerCallback(this);
    }

    @Override
    public void log(String logEntry) {
        lastEventLog.setText(logEntry);
        String logAppend = logger.getText().toString();
        if ("".equals(logger.getText().toString())) {
            logAppend = logEntry;
        } else {
            logAppend += "\n" + logEntry;
        }
        logger.setText(logAppend);
        if (scrollView != null) {
            scrollView.post(new Runnable() {
                @Override
                public void run() {
                    scrollView.fullScroll(View.FOCUS_DOWN);
                }
            });
        }
    }

    @Override
    public void onVideoPlayerCreated() {
        Log.i(TAG, "onVideoPlayerCreated");
    }

    @Override
    public void onResourceReady(MediaResource resource) {
        Log.i(TAG, "onResourceReady " + resource.getRedirectedUrl());
        mediaResource = resource;
        configVideoView();
        if (csaiManager != null)
            csaiManager.setVideoPlayerView(mVideoView);
        if (daiManager != null)
            daiManager.setVideoPlayerView(mVideoView);
        mVideoView.play(mediaResource);
    }

    @Override
    public void onResourceError(ErrorType errorType, Exception exception) {
        Log.e(TAG, "onResourceError");
    }

    private void configVideoView() {
        mVideoView = mVideoContainer.getVideoPlayer();
        mPlayerControlsOverlay.setVideoPlayerContainer(mVideoContainer);
        mVideoView.setFullScreenMode(VideoPlayerView.FULLSCREEN_MODE_KEEP_ASPECT_RATIO);
        mVideoView.setLicense(AMP_LICENSE);
        mVideoView.setKeepScreenOn(true);
        mVideoView.enableDVRfeatures(true);
        mVideoView.useContentTimeline(true);


        if (csaiManager != null) {
            csaiManager.addEventsListener(mPlayerControlsOverlay);
            mVideoView.setTimelineListener(csaiManager);
        }
        if (daiManager != null) {
            daiManager.addEventsListener(mPlayerControlsOverlay);
            mVideoView.setTimelineListener(daiManager);
        }
    }

    @Override
    public void onPause() {
        pausePlayer();
        super.onPause();
    }

    private void pausePlayer() {
        if (csaiManager != null)
            csaiManager.onPause();
        if (daiManager != null)
            daiManager.onPause();

    }

    @Override
    public void onResume() {
        resumePlayer();
        super.onResume();
    }

    private void resumePlayer() {
        if (csaiManager != null)
            csaiManager.onResume(true);
        if (daiManager != null)
            daiManager.onResume(true);
    }

    @Override
    public void onDestroy() {
        destroyPlayer();
        mVideoView = null;
        super.onDestroy();
    }

    private void destroyPlayer() {
        if (mVideoView != null) mVideoView.onDestroy();
        mPlayerControlsOverlay.removeVideoPlayerViewReference();
        csaiManager = null;
        daiManager = null;
    }

    /*ADS RELATED AMP EVENTS*/

    @Override
    public void onListenerRegistered() {

    }

    @Override
    public void onAdsError(String s) {
        Log.e(TAG, "onAdsError " + s);
    }

    @Override
    public void onPauseContentRequested() {
        Log.i(TAG, "onPauseContentRequested");
    }

    @Override
    public void onResumeContentRequested() {
        Log.i(TAG, "onResumeContentRequested");
    }

    @Override
    public void onAdsTapped() {
        Log.i(TAG, "onAdsTapped");
    }

    @Override
    public void onAdsInitialized() {
        Log.i(TAG, "onAdsInitialized");

        BaseManager adsManager = getBaseManager();
        if (adsManager == null) return;

        adsManager.addAdEventListener(adEvent -> Log.i("IMA-Events-Test", adEvent.toString()));
    }

    private BaseManager getBaseManager() {
        BaseManager adsManager = null;
        if (csaiManager != null) {
            adsManager = csaiManager.getManager();
        } else if (daiManager != null) {
            adsManager = daiManager.getManager();
        }
        return adsManager;
    }

    @Override
    public void onAdsLoaded(AdsCount adsCount) {
        Log.i(TAG, "onAdsLoaded " + adsCount);
    }

    @Override
    public void onAdsStarted(GoogleAdsInfo adsInfo) {
        Log.i(TAG, "onAdsStarted " + adsInfo);
    }

    @Override
    public void onAdsPaused() {
        Log.i(TAG, "onAdsPaused");
    }

    @Override
    public void onAdsResumed() {
        Log.i(TAG, "onAdsResumed");
    }

    @Override
    public void onAdBreakStarted() {
        Log.i(TAG, "onAdBreakStarted");
        mPlayerControlsOverlay.setVisibility(View.GONE);
    }

    @Override
    public void onAdBreakEnded() {
        Log.i(TAG, "onAdBreakEnded");
        mPlayerControlsOverlay.setVisibility(View.VISIBLE);
    }

    @Override
    public void onAdsEnded() {
        Log.i(TAG, "onAdsEnded");
    }

    @Override
    public void onAdEvent() {
        Log.i(TAG, "onAdsEvent");
    }

    @Override
    public void onAllPostrollsEnded() {
        Log.i(TAG, "onAllPostrollsEnded");
    }

    @Override
    public void onAdsTrackProgress(int i) {
        Log.i(TAG, "onAdsTrackProgress " + AdsHelper.translateProgressCode(i) + "%");
    }

    @Override
    public void onAdSkipped(){
        Log.i(TAG, "onAdSkipped");
    }

    @Override
    public void onAdsPlayheadUpdate(int i) {
        Log.i(TAG, "onAdsPlayheadUpdate " + i);
    }


    /**
     * ACTION BAR MENU OPTIONS
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.play_DAI_VOD:
                pausePlayer();
                setupDAI();
                setInlineUI();
                log("AMP/IMA: Loading SERVER SIDE VOD ... ");
                daiManager.setAdTagParameters(getDaiParams());
                daiManager.playWithServerAds("19463", "tears-of-steel");
                return true;

            case R.id.play_DAI_live:
                pausePlayer();
                setupDAI();
                setInlineUI();
                log("AMP/IMA: Loading SERVER SIDE LIVE ... ");
                daiManager.playWithServerAds("sN_IYUG8STe1ZzhIIE_ksA");
                return true;

            case R.id.play_CLIENT_VOD:
                pausePlayer();
                setupCSAI();
                setInlineUI();
                log("AMP/IMA: Loading CLIENT SIDE VOD ... ");
                csaiManager.setAdsUrl(ADS_URL);
                mVideoContainer.prepareResource(getString(R.string.stream_url));
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private Map<String, String> getDaiParams() {
        Map<String, String> imaParameters = new HashMap<>();
        //imaParameters.put("cust_params", "loc=homescreen"); //Providing this param, there will be no ads sent
        return imaParameters;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_main, menu);
        return true;
    }

    private void setInlineUI() {
        mPlayerControlsOverlay.overrideControlsLayout(R.layout.overlay_controls);
        mPlayerControlsOverlay.setShowControlsAtStart(false);
        setupInlinePlayerControls();
    }

    private void setupInlinePlayerControls() {
        mPlayerControlsOverlay.managePlayPause(R.id.androidsdk_playPauseCtrl,
                R.drawable.play,
                R.drawable.pause);
        mPlayerControlsOverlay.manageCurrentPosition(R.id.androidsdk_seekbarTextCtrl);
        mPlayerControlsOverlay.manageTimeRemaining(R.id.video_duration);
        mPlayerControlsOverlay.manageScrubbing(R.id.androidsdk_seekbarCtrl,
                R.id.androidsdk_seekToLiveAction);
    }

    private void setupCSAI() {
        destroyPlayer();
        ImaSdkSettings settings = ImaSdkFactory.getInstance().createImaSdkSettings();
        csaiManager = IMA.create(this)
                .withImaSdkSettings(settings)
                .buildClientSideManager();
        //csaiManager.setVastTimeout(8000);
        csaiManager.setLog(this);
        csaiManager.setVideoPlayerContainer(mVideoContainer);
        csaiManager.addEventsListener(this);
    }

    private void setupDAI() {
        destroyPlayer();
        ImaSdkSettings settings = ImaSdkFactory.getInstance().createImaSdkSettings();
        daiManager = IMA.create(this)
                .withImaSdkSettings(settings)
                .buildServerSideManager();

        daiManager.setLog(this);
        daiManager.setVideoPlayerContainer(mVideoContainer);
        daiManager.addEventsListener(this);
    }

}
